#baseDir<-"../../PhD/_paper_XyDye/imageAnalysis_supplement";source(paste0(baseDir,"/step3_classify.r"))

# computes blue stained area fractions and creates paper Fig.04

# requires:
#  config.r
#  images/02_masks/xx.png
#  images/03_distances/xx.png
#  images/04_NDBI/xx.png

# creates:
#  images/05_results/xx.png
#  fig04.pdf
#  result_step3_stainedAreaFractions.csv


library("readbitmap")
library("png")

if(!exists("baseDir")) baseDir <- getwd()
source(paste0(baseDir,"/config.r"))
experimentInfo <- read.table(paste0(baseDir,'/',config.measurementFile), sep='\t', header=TRUE)
cumL <- cumsum(experimentInfo$SegmentLength) - experimentInfo$SegmentLength[1]

NDBI_threshold <- config.NDBI_threshold
dClasses       <- config.distanceClasses

dCols <- c(NA,rev(hcl.colors(dClasses,"Plasma"))) # colors for the distance classes
dLwds <- c(NA,((1+dClasses):1)^0.6+1)*0.8 # line widths for the distance classes

bFrac <- bFracs <- NULL # variable to hold the blue stained area fractions

for(n in 1:nrow(experimentInfo)){
	
	cat("processing",experimentInfo$SegmentID[n],"...\n")

	
	filepath <- paste0(baseDir,"/images/02_masks/",experimentInfo$SegmentID[n],".png")
	
	# load the xylemm mask
	mask <- read.bitmap(filepath)
	if(length(dim(mask))==3) mask <- mask[,,1] # one channel is enough, as the mask should contain no color information
	
	mask[mask<0.3] <- NA
	
	# load the NDBI file
	ndbi <- read.bitmap(gsub("02_masks","03_NDBI",filepath))
	if(length(dim(ndbi))==3) ndbi <- ndbi[,,1]
	ndbi <- ndbi*2-1 # restore original NDBI (ranging from -1 to 1 instead of 0 to 1 as in the loaded file)
	
	# load the distance from xylem border file
	distance <- read.bitmap(gsub("02_masks","04_distances",filepath))
	if(length(dim(distance))==3) distance <- distance[,,1]
	
	d <- ceiling(distance*dClasses) # group distances into distance classes

	
	res <- ndbi*mask
	
	res[res <  NDBI_threshold] <- -1
	
	bFracN <- NULL
	for(i in 1:dClasses){
		sel <- res[d==i]		
		bFracN <- c(bFracN, sum(sel > -1, na.rm=TRUE)/sum(is.finite(res)))
	}
	bFracs <- rbind(bFracs, bFracN)
	
	bFrac <- c(bFrac, if(n==1) 1 else sum(res > -1, na.rm=TRUE)/sum(is.finite(res)))
			
	breaks <- c(-1,NDBI_threshold,1)

	png(gsub("02_masks","05_results",filepath), width=ncol(mask), height=nrow(mask), bg = "transparent")
		par(mar=c(0,0,0,0))
		image(t(res)[,nrow(mask):1], axes=FALSE, col=c("#dddddd","#0088ff"), breaks=breaks, xaxs='i', yaxs='i')			
		contour(t(d)[,nrow(mask):1], add=TRUE, nlevels=1+dClasses, col=dCols, lwd=dLwds*3, drawlabels=FALSE)				
	dev.off()
}

# write results into text file
colnames(bFracs) <- paste0("dClass_",1:dClasses)
write.table(round(bFracs,3), paste0(baseDir,"/result_step3_stainedAreaFractions.csv"), sep='\t', quote=FALSE, row.names=FALSE)


#--------------------fig04-----------------------------------------------
pdf(paste0(baseDir,"/fig04.pdf"), height=6/2.54, width=8.6/2.54)

	par(mar=c(2.5,2.5,6,0.6), mgp=c(1.4,0.4,0), las=1, tcl=-0.25, cex=0.7)
	plot(cumL/10, 100*bFrac, pch=NA, ylab="Stained area fraction[%]        ", xlab="Flow path length [cm]",
		xaxs='i', yaxs='i')
	abline(v=cumL/10, lty=2, col="#cccccc")
	for(n in 1:ncol(bFracs)){
		y0 <- 100*rowSums(cbind(0,bFracs)[,1:n,drop=FALSE])
		y1 <- 100*rowSums(bFracs[,1:n,drop=FALSE])
		polygon(c(cumL,rev(cumL))/10, c(y0, rev(y1)), col= dCols[n+1], border=NA)
	}

	yOffset <- c(-10,20)[(1:nrow(experimentInfo)%%2)+1]
	xOffset <- -25
	
	yVals <- 130+yOffset
	xVals <- (cumL+xOffset)/10
	
	arrows(xVals,yVals,xVals, 107, length=0, xpd=TRUE, lwd=0.5, xpd=TRUE)
	arrows(xVals,107,xVals-xOffset/10, 101, length=0.03, xpd=TRUE, lwd=0.5, xpd=TRUE)
	
	for(n in 1:nrow(experimentInfo)){
		pic <- read.bitmap(paste0(baseDir,"/images/05_results/",experimentInfo$SegmentID[n],".png"))
		w <- 55
		h <- w*nrow(pic)/ncol(pic)*0.3
		
		rasterImage(pic, xVals[n]-w/10, yVals[n]-h,
					     xVals[n]+w/10, yVals[n]+h, xpd=TRUE)
	}
	
	text(xVals,yVals+17,labels=sprintf("#%02d",1:nrow(experimentInfo)), cex=0.7,xpd=TRUE)
	text(85.5,160,labels="(a)",adj=c(0.5,1.2), font=2, xpd=TRUE)

	text(84,99,labels="(b)",adj=c(0.5,1.2), font=2)
	
	par(fig=c(0.88,1,0.18,0.60), new=TRUE)
	
	mar=par("mar")
	par(mar=c(0,0,0,0))
	plot.new()
	rect(0,0,2,1, col="white")
	points(rep(1,dClasses), rep(0.5,dClasses), cex=(dClasses:1)*3.5, col=dCols[-1], pch=20)	
	text(c(0.94,0.8,0.68,0.54,0.40,0.25), 0.5, dClasses:1, col=rep(c('white','black'), each=3), cex=0.8)
	text(0.4, 0.8, labels="Xylem\ndepth\nclasses", cex=0.75, xpd=TRUE)
	par(mar=mar)
	

dev.off()
